from typing import Optional, Sequence, Self

from sqlalchemy.orm import Mapped, mapped_column

from portal import db
from portal.model.Base import Base


class User(Base):
    name: Mapped[str] = mapped_column(unique=True, index=True)

    @classmethod
    def authenticate(cls, name: str, password: str) -> Optional[Self]:
        """Try to authenticate a session with the given username and password.

        :return: A "User" instance, if successful. None, otherwise.
        """
        return None  # TODO

    @classmethod
    def get_by_name(cls, name: str) -> Optional[Self]:
        """Return the user with the given name, if they exist."""
        return db.session.execute(
            db.select(cls).where(cls.name == name)
        ).scalar_one_or_none()

    @property
    def keys(self) -> Sequence["Key"]:
        """
        Get all keys created by this user
        """
        return (
            db.session.execute(db.select(Key).where(Key.user_uuid == self.uuid))
            .scalars()
            .all()
        )

    def delete_keys(self) -> None:
        """
        Delete all keys created by this user
        """
        db.session.execute(db.delete(Key).where(Key.user_uuid == self.uuid))
        db.session.commit()

    def to_dict(self) -> dict[str, str]:
        return {"uuid": self.uuid, "name": self.name}


from portal.model.Key import Key
